
// *************************************************************
// ***                FLASH-Manager                          ***
// *** Programme zum Lesen und Schreiben der EEPROM-Tabellen ***
// *************************************************************


CONST
  CODE_01 : Byte = 08;
  CODE_02 : Byte = 16;


CONST
  // Basisadresse der Tabelle: TBL1_Typ_Ziel_Zeiger
  ADR_TBL1_Typ_Ziel_Zeiger : Word = $0E000;

  // Basisadresse der Tabelle: TBL2_Zeiger_4K_Spur_Nr
  ADR_TBL2_Zeiger_4K_Spur_Nr : Word = $0E800;

  // Basisadresse der Tabelle: TBL_4K_SPI_Adressen
  ADR_TBL3_4K_SPI_TMP_Spur_Nr : Word = $0F800;

  // Adresse der Initialisierungskennung 01
  ADR_aeltester_4K_Speicher_im_I2C : Word = $0FFFD;

  // Adresse der Initialisierungskennung 01
  ADR_INIT_Kennung_01 : Word = $0FFFE;

  // Adresse der Initialisierungskennung 02
  ADR_INIT_Kennung_02 : Word = $0FFFF;


  // Typ_Ziel_Zeiger
  TZZ_AT24C512_DIR        : Byte = 1;
  TZZ_AT24C512_READ_WRITE : Byte = 2;
  TZZ_W25Q64_READ         : Byte = 3;



VAR
  FM_TEMP_TBL1_Typ_Ziel_Zeiger : Byte;




{
Basisadressen der Tabellen
==========================


(Adressbereich: von 0000H bis FFFFH )

Die Nutzung des EEPROMs ist in 16 Bloecken zu 4KByte aufgeteilt
Block_00 = 00000H = Z_AT24C512_DIR
Block_01 = 01000H = Z_AT24C512_DIR
Block_02 = 02000H = Z_AT24C512_DIR
Block_03 = 03000H = Z_AT24C512_DIR
Block_04 = 04000H = Z_AT24C512_DIR
Block_05 = 05000H = Z_AT24C512_WRITE
Block_06 = 06000H = Z_AT24C512_WRITE
Block_07 = 07000H = Z_AT24C512_WRITE
Block_08 = 08000H = Z_AT24C512_WRITE
Block_09 = 09000H = Z_AT24C512_WRITE
Block_0A = 0A000H = Z_AT24C512_WRITE
Block_0B = 0B000H = Z_AT24C512_WRITE
Block_0C = 0C000H = Z_AT24C512_WRITE
Block_0D = 0D000H = Z_AT24C512_WRITE
Block_0E = 0E000H = Z_AT24C512_TBL
Block_0F = 0F000H = Z_AT24C512_TBL

Die Tabellen befinden sich im EEPROM AT24C512 im Block_0E
ab der Adresse 0E000H (bis 0FFFFH)


1. Tabelle TBL_EE_Ziel_Zeiger

Basisadresse: 0E000H (Laenge 0800H bis E7FFH)
Groesse: 2048 Byte


2. Tabelle TBL_4K_Basis_Adressen

Basisadresse: 0E800H (Laenge 1000H bis F7FFH)
Groesse: 4096 Byte


3. Tabelle TBL_4K_SPI_Adressen

Basisadresse: 0F800H (Laenge 10 Word = 20 Byte)
Groesse: 20 Byte


4. Speicherzellen zur Initialisierung

Am Ende des Speicher des AT24C512 (letzten beiden Speicherzellen)
wir eine Konstante eingetragen. Sie dient als
Initialisierungskennung.

Basisaddressen: 0FFFEH und 0FFFFH

Konstanten: 0FFFEH = 08 Deimal
            0FFFFH = 15 Deimal

}






// **********************************
// ***                            ***
// *** Anzeigeroutinen auf SERIAL ***
// ***                            ***
// **********************************



       {

procedure Anzeige_Typ_Ziel_Zeiger;
begin

  SER0_String:='ZZ(';
  WordToHexString(Flash_Address);
  SER0_String:=SER0_String+Word_Hex_String;
  SER0_String:=SER0_String+'H)=';

  case FM_TEMP_TBL1_Typ_Ziel_Zeiger of

    TZZ_AT24C512_DIR : SER0_String:=SER0_String+'TZZ_AT24C512_DIR';
    |

    TZZ_AT24C512_READ_WRITE : SER0_String:=SER0_String+'TZZ_AT24C512_READ_WRITE';
    |

    TZZ_W25Q64_READ : SER0_String:=SER0_String+'TZZ_W25Q64_READ';
    |

  else

    SER0_String:=SER0_String+'kein Typ_Ziel_Zeiger ???';

  endcase;

  WriteLn_SER0;

end Anzeige_Typ_Ziel_Zeiger;






procedure Anzeige_TBL1_Typ_Ziel_Zeiger;
var
  Address_counter : Byte;
begin

  SER0_String:='*** TBL1_Typ_Ziel_Zeiger ***';
  WriteLn_SER0;

  Flash_Address:=ADR_TBL1_Typ_Ziel_Zeiger;

  for Address_counter:=0 to 49 do

    FM_TEMP_TBL1_Typ_Ziel_Zeiger:=AT_Data_Read_one_Byte;
    Anzeige_Typ_Ziel_Zeiger;

    INC(Flash_Address);

  endfor;

  SER0_String:='...';
  WriteLn_SER0;

end Anzeige_TBL1_Typ_Ziel_Zeiger;






procedure Anzeige_Basis_Adresse;
var
  TAB_Data_L : Byte;
  TAB_Data_H : Byte;
begin
    SER0_String:='BA(';
    WordToHexString(Flash_Address);
    SER0_String:=SER0_String+Word_Hex_String;
    SER0_String:=SER0_String+'H)=';

    TAB_Data_L:=AT_Data_Read_one_Byte;
    INC(Flash_Address);
    TAB_Data_H:=AT_Data_Read_one_Byte;
    INC(Flash_Address);

    ByteToHexString(TAB_Data_H);
    SER0_String:=SER0_String+Byte_Hex_String;
    ByteToHexString(TAB_Data_L);
    SER0_String:=SER0_String+Byte_Hex_String;
    SER0_String:=SER0_String+'H';

    WriteLn_SER0;

end Anzeige_Basis_Adresse;





procedure Anzeige_TBL2_Zeiger_4K_Spur_Nr;
var
  Address_counter : Byte;
begin

  SER0_String:='*** TBL2_Zeiger_4K_Spur_Nr ***';
  WriteLn_SER0;

  Flash_Address:=ADR_TBL2_Zeiger_4K_Spur_Nr;

  for Address_counter:=0 to 49 do
    Anzeige_Basis_Adresse;
  endfor;

  SER0_String:='...';
  WriteLn_SER0;

end Anzeige_TBL2_Zeiger_4K_Spur_Nr;






procedure Anzeige_TBL3_4K_SPI_TMP_Spur_Nr;
var
  Address_counter : Byte;
  TAB_Data_L : Byte;
  TAB_Data_H : Byte;
begin

  SER0_String:='*** TBL3_4K_SPI_TMP_Spur_Nr ***';
  WriteLn_SER0;

  Flash_Address:=ADR_TBL3_4K_SPI_TMP_Spur_Nr;

  for Address_counter:=0 to 9 do

    SER0_String:='SPI_Adr(';
    WordToHexString(Flash_Address);
    SER0_String:=SER0_String+Word_Hex_String;
    SER0_String:=SER0_String+'H,';
    ByteToHexString(Address_counter);
    SER0_String:=SER0_String+Byte_Hex_String;
    SER0_String:=SER0_String+'H)=';


    TAB_Data_L:=AT_Data_Read_one_Byte;
    INC(Flash_Address);
    TAB_Data_H:=AT_Data_Read_one_Byte;
    INC(Flash_Address);


    ByteToHexString(TAB_Data_H);
    SER0_String:=SER0_String+Byte_Hex_String;
    ByteToHexString(TAB_Data_L);
    SER0_String:=SER0_String+Byte_Hex_String;
    SER0_String:=SER0_String+'H';


    WriteLn_SER0;


  endfor;

  SER0_String:='...';
  WriteLn_SER0;


  Flash_Address:=ADR_aeltester_4K_Speicher_im_I2C;

  SER0_String:='aeltester_4K_Speicher_im_I2C=';
  TAB_Data_L:=AT_Data_Read_one_Byte;
  ByteToHexString(TAB_Data_L);
  SER0_String:=SER0_String+Byte_Hex_String;
  WriteLn_SER0;

  SER0_String:='...';
  WriteLn_SER0;


end Anzeige_TBL3_4K_SPI_TMP_Spur_Nr;


              }




// *****************************************************
// ***                                               ***
// *** Initialisierung der Tabellen und des AT24C512 ***
// ***                                               ***
// *****************************************************



procedure INIT_TBL1_Typ_Ziel_Zeiger;
var
  Address_counter : Word;
begin


// Laenge 4
  for Address_counter:=0 to 3 do
    Sektor_Puffer[Address_counter]:=TZZ_AT24C512_DIR;
  endfor;

  // Laenge 10
  for Address_counter:=4 to 13 do
    Sektor_Puffer[Address_counter]:=TZZ_AT24C512_READ_WRITE;
  endfor;

  // Rest
  for Address_counter:=14 to 127 do
    Sektor_Puffer[Address_counter]:=TZZ_W25Q64_READ;
  endfor;

  Flash_Address:=ADR_TBL1_Typ_Ziel_Zeiger;
  // Schreiben der EE_Ziel_Zeiger
  AT_Write_Data_Array_Wait;


  for Address_counter:=0 to 127 do
    Sektor_Puffer[Address_counter]:=TZZ_W25Q64_READ;
  endfor;


  for Address_counter:=1 to 15 do

    Flash_Address:=Flash_Address+128;

    // Schreiben der EE_Ziel_Zeiger
    AT_Write_Data_Array_Wait;

  endfor;


end INIT_TBL1_Typ_Ziel_Zeiger;







procedure INIT_TBL2_Zeiger_4K_Spur_Nr;
var
  Address_counter : Word;
  Address_counter2 : Word;
  Address_counter3 : Word;
  Address_Wert : Word;
  b1 : Byte;
begin

  Address_Wert:=0;
  Address_counter2:=0;
  for Address_counter:=0 to 13 do
    b1:=Low(Address_Wert);
    Sektor_Puffer[Address_counter2]:=b1;
    Address_counter2:=Address_counter2+1;
    b1:=High(Address_Wert);
    Sektor_Puffer[Address_counter2]:=b1;
    Address_counter2:=Address_counter2+1;
    INC(Address_Wert);
  endfor;


  for Address_counter:=14 to 127 do
    b1:=Low(Address_Wert);
    Sektor_Puffer[Address_counter2]:=b1;
    Address_counter2:=Address_counter2+1;
    b1:=High(Address_Wert);
    Sektor_Puffer[Address_counter2]:=b1;
    Address_counter2:=Address_counter2+1;
    INC(Address_Wert);
  endfor;


  Flash_Address:=ADR_TBL2_Zeiger_4K_Spur_Nr;
  // Schreiben der 4K_Basis_Adressen
  AT_Write_Data_Array_Wait;




  for Address_counter:=1 to 31 do

    Address_counter2:=0;
    for Address_counter3:=0 to 127 do
      b1:=Low(Address_Wert);
      Sektor_Puffer[Address_counter2]:=b1;
      Address_counter2:=Address_counter2+1;
      b1:=High(Address_Wert);
      Sektor_Puffer[Address_counter2]:=b1;
      Address_counter2:=Address_counter2+1;
      INC(Address_Wert);
    endfor;

    Flash_Address:=Flash_Address+128;

    // Schreiben der EE_Ziel_Zeiger
    AT_Write_Data_Array_Wait;

  endfor;


end INIT_TBL2_Zeiger_4K_Spur_Nr;







procedure INIT_TBL3_4K_SPI_TMP_Spur_Nr;
var
  Address_Wert : Byte;
begin

  Flash_Address:=ADR_TBL3_4K_SPI_TMP_Spur_Nr;

  for Address_Wert:=4 to 13 do

    AT_Data_Write_one_Byte_Wait(Address_Wert); // niederwertiges Byte
    INC(Flash_Address);
    AT_Data_Write_one_Byte_Wait(0); // hoeherwertiges Byte (bei 4..14 immer 0)
    INC(Flash_Address);

  endfor;


  Flash_Address:=ADR_aeltester_4K_Speicher_im_I2C;
  AT_Data_Write_one_Byte_Wait(0);


end INIT_TBL3_4K_SPI_TMP_Spur_Nr;






procedure INIT_Loeschen_und_Formatieren;
begin

  Format_DIR;
  INIT_TBL1_Typ_Ziel_Zeiger;
  INIT_TBL2_Zeiger_4K_Spur_Nr;
  INIT_TBL3_4K_SPI_TMP_Spur_Nr;


  // W25Q64 Loeschen !!!
  // FLASH_Chip_Erase;


  Flash_Address:=ADR_INIT_Kennung_01;
  AT_Data_Write_one_Byte_Wait(CODE_01);

  Flash_Address:=ADR_INIT_Kennung_02;
  AT_Data_Write_one_Byte_Wait(CODE_02);


end INIT_Loeschen_und_Formatieren;





procedure INIT_FLASH_Manager;
VAR
  b1 : Byte;
begin


  // 1. Lesen der Initialisierungskennung
  Flash_Address:=ADR_INIT_Kennung_01;
  b1:=AT_Data_Read_one_Byte;



  if b1=CODE_01 then


    // 2. Lesen der Initialisierungskennung
    Flash_Address:=ADR_INIT_Kennung_02;
    b1:=AT_Data_Read_one_Byte;



    if b1=CODE_02 then

    else

      INIT_Loeschen_und_Formatieren;

    endif;



  else

    INIT_Loeschen_und_Formatieren;

  endif;





end INIT_FLASH_Manager;








// *******************************
// ***                         ***
// *** FLASH-Manager Programme ***
// ***                         ***
// *******************************

// *******************************
// Hilfsprogramme und Funktionen
// *******************************

// Read_Typ_Ziel_Zeiger
procedure FM_Read_TBL1_Typ_Ziel_Zeiger(RD_4K_Spur : Word);
begin

  // Lesen des TBL1_Typ_Ziel_Zeiger
  Flash_Address:=ADR_TBL1_Typ_Ziel_Zeiger+RD_4K_Spur;
  FM_TEMP_TBL1_Typ_Ziel_Zeiger:=AT_Data_Read_one_Byte;

end FM_Read_TBL1_Typ_Ziel_Zeiger;




procedure FM_Write_TBL1_Typ_Ziel_Zeiger(WR_4K_Spur : Word);
begin

  // Schreiben des TBL1_Typ_Ziel_Zeiger
  Flash_Address:=ADR_TBL1_Typ_Ziel_Zeiger+WR_4K_Spur;
  AT_Data_Write_one_Byte_Wait(FM_TEMP_TBL1_Typ_Ziel_Zeiger);

end FM_Write_TBL1_Typ_Ziel_Zeiger;





// Read_TBL2_4K_Basis_Zeiger
function FM_Read_TBL2_4K_Spur_Nr(RD_4K_Spur : Word) : Word;
var
  RD_4K_BA_Word_L : Word;
  RD_4K_BA_Word_H : Word;
  RD_4K_BA_Byte : Byte;
begin

  // Read_TBL2_4K_Basis_Adressen
  Flash_Address:=RD_4K_Spur*2;
  Flash_Address:=Flash_Address+ADR_TBL2_Zeiger_4K_Spur_Nr;
  RD_4K_BA_Byte:=AT_Data_Read_one_Byte;
  RD_4K_BA_Word_L:=ByteToWord(RD_4K_BA_Byte);
  INC(Flash_Address);
  RD_4K_BA_Byte:=AT_Data_Read_one_Byte;
  RD_4K_BA_Word_H:=ByteToWord(RD_4K_BA_Byte);
  RD_4K_BA_Word_H:=RD_4K_BA_Word_H*256;
  RD_4K_BA_Word_H:=RD_4K_BA_Word_H+RD_4K_BA_Word_L;

  Result:=RD_4K_BA_Word_H;

end FM_Read_TBL2_4K_Spur_Nr;







procedure FM_Write_TBL2_4K_Spur_Nr(WR_4K_Spur, WR_4K_Inhalt : Word);
var
  b1 : Byte;
begin

  // Abspeichern des 4K-Speicher-Zeigers in der TBL_4K_Basis_Adressen
  Flash_Address:=WR_4K_Spur*2;
  Flash_Address:=Flash_Address+ADR_TBL2_Zeiger_4K_Spur_Nr;
  b1:=Low(WR_4K_Inhalt);

  AT_Data_Write_one_Byte_Wait(b1);
  INC(Flash_Address);
  b1:=High(WR_4K_Inhalt);
  AT_Data_Write_one_Byte_Wait(b1);

end FM_Write_TBL2_4K_Spur_Nr;






//
function FM_4K_Spur_und_Sektor_in_FLASH_Zeiger(ZO_4K_Spur : Word) : DWord;
var
  ZO_4K_Zeiger : DWord;
  ZO_4K_Sektor : DWord;
begin

  // entsteht aus 4096 / 128 = 32
  ZO_4K_Zeiger:=WordToDWord(ZO_4K_Spur);
  ZO_4K_Zeiger:=ZO_4K_Zeiger*32;
  ZO_4K_Sektor:=WordToDWord(STD_Sektor_Nr);
  ZO_4K_Zeiger:=ZO_4K_Zeiger+ZO_4K_Sektor;
  ZO_4K_Zeiger:=ZO_4K_Zeiger*128; // wg. Anzahl Bytes im Sektor

  Result:=ZO_4K_Zeiger;

end FM_4K_Spur_und_Sektor_in_FLASH_Zeiger;





// Read_aeltester_4K_Speicher_im_I2C
function FM_Read_aeltester_4K_Speicher_im_I2C : Byte;
var
  RD_aeltester_4K_Speicher_im_I2C : Byte;
begin

  // lesen des Zeigers der aeltesten 4K_Speicher-Adresse
  Flash_Address:=ADR_aeltester_4K_Speicher_im_I2C;
  RD_aeltester_4K_Speicher_im_I2C:=AT_Data_Read_one_Byte;

  Result:=RD_aeltester_4K_Speicher_im_I2C;

end FM_Read_aeltester_4K_Speicher_im_I2C;




// Write_aeltester_4K_Speicher_im_I2C
procedure FM_Write_aeltester_4K_Speicher_im_I2C(
                        WR_aeltester_4K_Speicher_im_I2C : Byte);
begin

  // schreiben des Zeigers der aeltesten 4K_Speicher-Adresse
  Flash_Address:=ADR_aeltester_4K_Speicher_im_I2C;
  AT_Data_Write_one_Byte_Wait(WR_aeltester_4K_Speicher_im_I2C);

end FM_Write_aeltester_4K_Speicher_im_I2C;






function FM_Read_Spur_Nr_aus_aeltester_4K_Speicher_im_I2C : Word;
var
  Read_Spur_Nr : Word;
  Zeiger_aeltester_4K_Speicher : Byte;
  w1 : Word;
  b1 : Byte;
begin

  Zeiger_aeltester_4K_Speicher:=FM_Read_aeltester_4K_Speicher_im_I2C;

  // Ermitteln der Ziel SPI-Adresse aus der Tabelle
  // ADR_TBL_4K_SPI_Adressen
  w1:=ByteToWord(Zeiger_aeltester_4K_Speicher);
  SHL(w1); // Zeiger * 2 -> Word-Adresse
  Flash_Address:=ADR_TBL3_4K_SPI_TMP_Spur_Nr;
  Flash_Address:=Flash_Address+w1;

  // lesen des Eintrags in der ADR_TBL3_4K_SPI_TMP_Spur_Nr
  // und Berechnen der SPI-Zieladresse
  b1:=AT_Data_Read_one_Byte;
  w1:=ByteToWord(b1);
  INC(Flash_Address);
  b1:=AT_Data_Read_one_Byte;
  Read_Spur_Nr:=ByteToWord(b1);
  Read_Spur_Nr:=Read_Spur_Nr*256;
  Read_Spur_Nr:=Read_Spur_Nr+w1;

  Result:=Read_Spur_Nr;

end FM_Read_Spur_Nr_aus_aeltester_4K_Speicher_im_I2C;






// Read_aeltester_4K_Speicher_im_I2C
function FM_SPI_Zieladresse_aus_aeltester_4K_Speicher : DWord;
var
  w1 : Word;
  SPI_Adresse_DWord : DWord;
begin

  w1:=FM_Read_Spur_Nr_aus_aeltester_4K_Speicher_im_I2C;

  SPI_Adresse_DWord:=WordToDWord(w1);
  SPI_Adresse_DWord:=SPI_Adresse_DWord*4096;

  Result:=SPI_Adresse_DWord;

end FM_SPI_Zieladresse_aus_aeltester_4K_Speicher;








//
procedure FM_Flash_Address_I2C_Quelladresse_aus_aeltester_4K_Speicher;
var
  Zeiger_aeltester_4K_Speicher : Byte;
begin

  // lesen des Zeigers der aeltesten 4K_Speicher-Adresse
  Zeiger_aeltester_4K_Speicher:=FM_Read_aeltester_4K_Speicher_im_I2C;

  // berechnen der I2C-Quelladresse
  Zeiger_aeltester_4K_Speicher:=Zeiger_aeltester_4K_Speicher+4; // DIR Offset
  Flash_Address:=ByteToWord(Zeiger_aeltester_4K_Speicher);
  Flash_Address:=Flash_Address*4096;

end FM_Flash_Address_I2C_Quelladresse_aus_aeltester_4K_Speicher;
















// *******************************
// Read und Write-Hauptprogramme
// *******************************



procedure FM_Read_Data_Array;
var
  FM_4K_Zeiger : Word;
  FLASH_Zeiger : DWord;
begin

  // 1. Lesen des Typ_Ziel_Zeiger
  FM_Read_TBL1_Typ_Ziel_Zeiger(STD_Spur_Nr);
  // Ergebnis in: FM_TEMP_TBL1_Typ_Ziel_Zeiger



  // 2. Lesen der TBL2_Zeiger_4K_Spur_Nr
  FM_4K_Zeiger:=FM_Read_TBL2_4K_Spur_Nr(STD_Spur_Nr);


  // 3. Uebergebener STD_Spur_Nr und STD_Sektor_Nr
  FLASH_Zeiger:=FM_4K_Spur_und_Sektor_in_FLASH_Zeiger(FM_4K_Zeiger);



  if FM_TEMP_TBL1_Typ_Ziel_Zeiger=TZZ_W25Q64_READ then

    // Lesen vom SPI-EEPROM W25Q64

    ADR_Byte_0:=DWordToByte(0,FLASH_Zeiger);
    ADR_Byte_1:=DWordToByte(1,FLASH_Zeiger);
    ADR_Byte_2:=DWordToByte(2,FLASH_Zeiger);


    W25Q64_Read_one_Sector;


  else


    // Lesen vom I2C-EEPROM AT24C512
    Flash_Address:=DWLow(FLASH_Zeiger);

    AT_Read_Data_Array;

  endif;


end FM_Read_Data_Array;







// Zurukschreiben des aleltesten 4K-Sektors
// vom I2C AT24C512 in den SPI W25Q64
// um den temporaeren Sektor wieder frei zu machen
procedure FM_Zurueckschreiben_4K_Sektor;
var
  SPI_Adresse_DWord : DWord;
  b1 : Byte;
  Z_Flash_Address : Word;
  Read_Spur_Nr : Word;
  Read_Spur_Nr2 : Word;
begin


  //
  FM_Flash_Address_I2C_Quelladresse_aus_aeltester_4K_Speicher;
  Z_Flash_Address:=Flash_Address;


  SPI_Adresse_DWord:=FM_SPI_Zieladresse_aus_aeltester_4K_Speicher;

  ADR_Byte_0:=DWordToByte(0,SPI_Adresse_DWord);
  ADR_Byte_1:=DWordToByte(1,SPI_Adresse_DWord);
  ADR_Byte_2:=DWordToByte(2,SPI_Adresse_DWord);

  // Loeschen des 4K-Abschnittes im SPI-W25Q64
  FLASH_Sector_Erase;


  Flash_Address:=Z_Flash_Address;


  for b1:=0 to 31 do


    // Lesen jeweils eines Sektors (128 Byte) vom I2C-AT24C512
    // in den Sektor_Puffer
    AT_Read_Data_Array;
    Flash_Address:=Flash_Address+128;


    // und dann Schreiben
    // des Sektor_Puffers in den SPI-W25Q64
    ADR_Byte_0:=DWordToByte(0,SPI_Adresse_DWord);
    ADR_Byte_1:=DWordToByte(1,SPI_Adresse_DWord);
    ADR_Byte_2:=DWordToByte(2,SPI_Adresse_DWord);

    W25Q64_Write_one_Sector;

    SPI_Adresse_DWord:=SPI_Adresse_DWord+128;

  endfor;


  // lesen der Spurnummer aus aeltester_4K_Speicher_im_I2C
  Read_Spur_Nr:=FM_Read_Spur_Nr_aus_aeltester_4K_Speicher_im_I2C;


  // Abspeichern des Typ_Ziel_Zeiger in der TBL1_Typ_Ziel_Zeiger
  FM_TEMP_TBL1_Typ_Ziel_Zeiger:=TZZ_W25Q64_READ;
  FM_Write_TBL1_Typ_Ziel_Zeiger(Read_Spur_Nr);

  Read_Spur_Nr2:=Read_Spur_Nr;

  // Abspeichern Tabelle 2 Read_Spur_Nr an Read_Spur_Nr
  FM_Write_TBL2_4K_Spur_Nr(Read_Spur_Nr,Read_Spur_Nr2);


end FM_Zurueckschreiben_4K_Sektor;








// Verschieben 4K_Sektor vom SPI-W25W64
// in den 4K-Speicher im I2C-AT24C512
// an der Stelle des aeltesten 4K-Speichers
procedure FM_Verschieben_4K_Sektor;
var
  SPI_Adresse_DWord : DWord;
  Zeiger_aeltester_4K_Speicher : Byte;
  Zeiger_aeltester_4K_Speicher_Word : Word;
  w1 : Word;
  b1 : Byte;
begin

  // lesen des Zeigers der aeltesten 4K_Speicher-Adresse
  Zeiger_aeltester_4K_Speicher:=FM_Read_aeltester_4K_Speicher_im_I2C;

  // berechnen der I2C-Zieladresse
  Zeiger_aeltester_4K_Speicher:=Zeiger_aeltester_4K_Speicher+4; // DIR Offset
  Flash_Address:=ByteToWord(Zeiger_aeltester_4K_Speicher);
  Flash_Address:=Flash_Address*4096;

  // berechnen der SPI-Quelladresse
  SPI_Adresse_DWord:=WordToDWord(STD_Spur_Nr);
  SPI_Adresse_DWord:=SPI_Adresse_DWord*4096;

  for b1:=0 to 31 do

    // Lesen jeweils eines Sektors (128 Byte) vom SPI-W25Q64
    // in den Sektor_Puffer
    ADR_Byte_0:=DWordToByte(0,SPI_Adresse_DWord);
    ADR_Byte_1:=DWordToByte(1,SPI_Adresse_DWord);
    ADR_Byte_2:=DWordToByte(2,SPI_Adresse_DWord);

    W25Q64_Read_one_Sector;

    SPI_Adresse_DWord:=SPI_Adresse_DWord+128;

    // und dann Schreiben
    // des Sektor_Puffers in den I2C-AT24C512
    AT_Write_Data_Array_Wait;
    Flash_Address:=Flash_Address+128;

  endfor;



  // Aendern des Eintrages in den beiden Tabellen
  // ============================================


  // Abspeichern des 4K-Ziel_Zeigers in der TBL1_Typ_Ziel_Zeiger
  FM_TEMP_TBL1_Typ_Ziel_Zeiger:=TZZ_AT24C512_READ_WRITE;
  FM_Write_TBL1_Typ_Ziel_Zeiger(STD_Spur_Nr);


  // Abspeichern des 4K-Speicher-Zeigers in der TBL2_4K_Spur_Nr
  Zeiger_aeltester_4K_Speicher:=FM_Read_aeltester_4K_Speicher_im_I2C;
  w1:=ByteToWord(Zeiger_aeltester_4K_Speicher);
  w1:=w1+4;
  FM_Write_TBL2_4K_Spur_Nr(STD_Spur_Nr,w1);



  // Ermitteln der Ziel SPI-Adresse aus der Tabelle
  // ADR_TBL_4K_SPI_Adressen
  w1:=ByteToWord(Zeiger_aeltester_4K_Speicher);
  SHL(w1); // Zeiger * 2 -> Word-Adresse
  Flash_Address:=ADR_TBL3_4K_SPI_TMP_Spur_Nr;
  Flash_Address:=Flash_Address+w1;


  // Abspeichern der neuen 4K-Adresse
  b1:=Low(STD_Spur_Nr);
  AT_Data_Write_one_Byte_Wait(b1);
  INC(Flash_Address);
  b1:=High(STD_Spur_Nr);
  AT_Data_Write_one_Byte_Wait(b1);



  // Ermitteln den naechsten aeltester_4K_Speicher_im_I2C
  INC(Zeiger_aeltester_4K_Speicher);
  if Zeiger_aeltester_4K_Speicher>9 then
    Zeiger_aeltester_4K_Speicher:=0;
  endif;


  // Abspeichern des neuen aeltester_4K_Speicher_im_I2C
  FM_Write_aeltester_4K_Speicher_im_I2C(Zeiger_aeltester_4K_Speicher);


end FM_Verschieben_4K_Sektor;










procedure FM_Test_Verschieben_Spur;
begin

  // 1. Lesen des Typ_Ziel_Zeiger
  FM_Read_TBL1_Typ_Ziel_Zeiger(STD_Spur_Nr);
  // Ergebnis in: FM_TEMP_TBL1_Typ_Ziel_Zeiger

  // 2. wenn noetig, Verschieben des 4K-Sektors
  if FM_TEMP_TBL1_Typ_Ziel_Zeiger=TZZ_W25Q64_READ then

    FM_Zurueckschreiben_4K_Sektor;
    FM_Verschieben_4K_Sektor;

  endif;

end FM_Test_Verschieben_Spur;









procedure FM_Write_Data_Array;
var
  FM_4K_Spur : Word;
  FLASH_Zeiger : DWord;
begin

  // 1. Lesen der TBL_4K_Basis_Adressen
  FM_4K_Spur:=FM_Read_TBL2_4K_Spur_Nr(STD_Spur_Nr);

  // 2. Berechnung FLASH-Zeiger
  FLASH_Zeiger:=FM_4K_Spur_und_Sektor_in_FLASH_Zeiger(FM_4K_Spur);

  // 3. Lesen vom I2C-EEPROM AT24C512
  Flash_Address:=DWLow(FLASH_Zeiger);

  // 4. Schreiben Data_Array
  AT_Write_Data_Array_Wait;


end FM_Write_Data_Array;








