
CONST

  // 0 1 0 1 0 1 0 0 -> $54 - bei A2 = 1
//  ADR_AT24C512 : Byte = $54;


  // 0 1 0 1 0 0 0 0 -> $50 - bei A2 = 0
  ADR_AT24C512 : Byte = $50;



// ******************************
// *** Hardware PINs fuer I2C ***
// ******************************
VAR
{
D0 - RXD0      = E
D1 - TXD0      = A / L
D2 - SCL - I2C = E
D3 - SDA - I2C = E
D4 - RD_Z80    = E
D5 - WR_Z80    = E
D6 - RESET_Z80 = A / H
D7 - CLK_Z80   = A / L
}
  SCL_I2C_A[PortD,2] : bit; // A
  SCL_I2C_E[PIND,2]  : bit; // E
  SCL_I2C_D[DDRD,2]  : bit; // D

  SDA_I2C_A[PortD,3] : bit; // A
  SDA_I2C_E[PIND,3]  : bit; // E
  SDA_I2C_D[DDRD,3]  : bit; // D



VAR
 // I2C_Sector, I2C_Track : Byte;
  Flash_Address : Word;


// ********************************************************





// *******************************
// *** hardwarenahe Prozeduren ***
// *******************************




procedure SDA_Set;
begin
  SetBit(SDA_I2C_A);
end SDA_Set;


procedure SCL_Set;
begin
  SetBit(SCL_I2C_A);
end SCL_Set;


procedure SDA_Clear;
begin
  ClearBit(SDA_I2C_A);
end SDA_Clear;


procedure SCL_Clear;
begin
  ClearBit(SCL_I2C_A);
end SCL_Clear;


procedure SDA_OUT;
begin
  SetBit(SDA_I2C_D);
end SDA_OUT;


procedure SDA_IN;
begin
  ClearBit(SDA_I2C_D);
end SDA_IN;



procedure SCL_OUT;
begin
  SetBit(SCL_I2C_D);
end SCL_OUT;



function SDA_Read : Boolean;
var
  b1 : Boolean;
begin

  if BitHigh(SDA_I2C_E) then
    b1:=true;
  else
    b1:=false;
  endif;

  Result:=b1;

end SDA_Read;







// ****************************
// *** I2C Basis Prozeduren ***
// ****************************





procedure Init_I2C;
begin

  SDA_Set;
  SCL_Set;

  SDA_OUT;
  SCL_OUT;

  SDA_Set;
  SCL_Set;

  Flash_Address:=0;

end Init_I2C;









// Read one Byte from Slave, MSB fist, send NACK
function ds_Shift_In_NACK : Byte;
var
  ds_data   : Byte;
  ds_couter : Byte;
begin

  // Data as Input
  SDA_IN;

  ds_data:=0;

  for ds_couter:=0 to 7 do

    SHL(ds_data);

    if SDA_Read then
      ds_data:=ds_data or %00000001;
    endif;

    SCL_Set;

    SCL_Clear;

  endfor;

  // Data as Output
  SDA_OUT;
  SDA_Set;

  SCL_Set;

  SCL_Clear;

  // Data as Output
  SDA_Clear;

  Result:=ds_data;

end ds_Shift_In_NACK;





// Read one Byte from Slave, MSB fist, send ACK
function ds_Shift_In_ACK : Byte;
var
  ds_data   : Byte;
  ds_couter : Byte;
begin

  // Data as Input
  SDA_IN;

  ds_data:=0;

  for ds_couter:=0 to 7 do

    SHL(ds_data);

    if SDA_Read then
      ds_data:=ds_data or %00000001;
    endif;

    SCL_Set;

    SCL_Clear;

  endfor;

  // Data as Output
  SDA_Clear;
  SDA_OUT;
  SDA_Clear;

  SCL_Set;

  SCL_Clear;

  Result:=ds_data;

end ds_Shift_In_ACK;





// Write one Byte to Slave, MSB fist
function ds_Shift_Out(ds_data : Byte) : Boolean;
var
  ds_couter : Byte;
  ds_result : Boolean;
  ds_data_2 : Byte;
begin

  // Data as Output
  SDA_Clear;
  SDA_OUT;
  SDA_Clear;

  for ds_couter:=0 to 7 do

    ds_data_2:=ds_data and %10000000;

    if ds_data_2 = 0 then
      SDA_Clear;
    else
      SDA_Set;
    endif;

    SHL(ds_data);

    SCL_Set;

    SCL_Clear;

  endfor;

  // Data as Input
  SDA_IN;

  SCL_Set;

  // Ergebnis ist true, wenn ACK=0 war
  if SDA_Read then
    ds_result:=false;
  else
    ds_result:=true;
  endif;

  SCL_Clear;

  // Data as Output
  SDA_Clear;
  SDA_OUT;
  SDA_Clear;

  Result:=ds_result;

end ds_Shift_Out;







// Write - Device-Address to Slave
function I2C_Device_Address_to_Write(I2C_Address : Byte) : Boolean;
var
  ds_result : Boolean;
begin

  SHL(I2C_Address);

  ds_result:=ds_Shift_Out(I2C_Address);
  Result:=ds_result;

end I2C_Device_Address_to_Write;







// Read - Device-Address to Slave
function I2C_Device_Address_to_Read(I2C_Address : Byte) : Boolean;
var
  ds_result : Boolean;
begin

  SHL(I2C_Address);
  I2C_Address:=I2C_Address or %00000001;

  ds_result:=ds_Shift_Out(I2C_Address);
  Result:=ds_result;

end I2C_Device_Address_to_Read;






// *******************************
// *** I2C Speicher-Prozeduren ***
// *******************************


// Hilfsprozedur zum Setzen der internen Speicheradresse
procedure AT_Data_Write_Word_Address;
var
  b1 : Byte;
  Address_ACK : Boolean;
begin

  // Start-Sequenz
  SDA_Clear;

  SCL_Clear;



  Address_ACK:=I2C_Device_Address_to_Write(ADR_AT24C512);

  b1:=High(Flash_Address);

  Address_ACK:=ds_Shift_Out(b1);

  b1:=Low(Flash_Address);

  Address_ACK:=ds_Shift_Out(b1);

  // Stop-Sequenz

  SCL_Set;

  SDA_Set;

end AT_Data_Write_Word_Address;





// ****************************
// *** 2. ARRAY-Operationen ***
// ****************************






// liest ein Byte aus dem I2C-Flasch (Ergebnis ist der Rueckgabewert)
function AT_Data_Read_one_Byte : Byte;
var
  ds_data : Byte;
  Address_ACK : Boolean;
begin



  // LED-Weiss_AT24C512 ein
  SetBit(LED_Weiss_AT24C512);
  LED_Weiss_AT24C512_Zaehler:=LED_Weiss_AT24C512_Zaehler_Ende;




  // setzen der Adresse Flash_Address
  // als Startadresse
  Address_ACK:=AT_Data_Write_Word_Address;


  // Start-Sequenz
  SDA_Clear;

  SCL_Clear;


  // I2C Address
  Address_ACK:=I2C_Device_Address_to_Read(ADR_AT24C512);

  ds_data:=ds_Shift_In_NACK;


  // Stop-Sequenz
  SCL_Set;

  SDA_Set;

  Result:=ds_data;

end AT_Data_Read_one_Byte;







// liest 128 Byte aus dem Flash ab der RAM_Address in das Daten_Array
procedure AT_Read_Data_Array;
var
  rba_data : Byte;
  Address_ACK : Boolean;
  rba_counter : Word;

begin



  // LED-Weiss_AT24C512 ein
  SetBit(LED_Weiss_AT24C512);
  LED_Weiss_AT24C512_Zaehler:=LED_Weiss_AT24C512_Zaehler_Ende;





  Address_ACK:=AT_Data_Write_Word_Address;

  // Start-Sequenz
  SDA_Clear;

  SCL_Clear;


  // I2C Address
  Address_ACK:=I2C_Device_Address_to_Read(ADR_AT24C512);

  for rba_counter:=0 to 126 do

    rba_data:=ds_Shift_In_ACK;
    Sektor_Puffer[rba_counter]:=rba_data;

  endfor;

  rba_data:=ds_Shift_In_NACK;
  Sektor_Puffer[127]:=rba_data;


  // Stop-Sequenz
  SCL_Set;

  SDA_Set;

end AT_Read_Data_Array;











// schreibt ein Byte in den I2C-Flasch mit Wartezeit
procedure AT_Data_Write_one_Byte_Wait(Data_Byte : Byte);
var
  Address_ACK : Boolean;
  b1 : Byte;
begin



  // LED-Weiss_AT24C512 ein
  SetBit(LED_Weiss_AT24C512);
  LED_Weiss_AT24C512_Zaehler:=LED_Weiss_AT24C512_Zaehler_Ende;




  // Start-Sequenz
  SDA_Clear;

  SCL_Clear;

  // I2C Address
  Address_ACK:=I2C_Device_Address_to_Write(ADR_AT24C512);

  b1:=High(Flash_Address);

  Address_ACK:=ds_Shift_Out(b1);

  b1:=Low(Flash_Address);

  Address_ACK:=ds_Shift_Out(b1);

  Address_ACK:=ds_Shift_Out(Data_Byte);

  // Stop-Sequenz
  SCL_Set;

  SDA_Set;




  // *******

  // Start-Sequenz
  SDA_Clear;
  SCL_Clear;

  // Acknowledge Polling (Datenblatt Seite 11)
  Address_ACK:=I2C_Device_Address_to_Write(ADR_AT24C512);


  // Stop-Sequenz
  SCL_Set;

  SDA_Set;


  while Address_ACK=false do

    // Start-Sequenz
    SDA_Clear;
    SCL_Clear;

    // I2C Address
    Address_ACK:=I2C_Device_Address_to_Write(ADR_AT24C512);

    // Stop-Sequenz
    SCL_Set;

    SDA_Set;


  endwhile;





end AT_Data_Write_one_Byte_Wait;











// schreibt 128 Byte aus dem Daten_Array in den Flash
procedure AT_Write_Data_Array_Wait;
var
  Address_ACK : Boolean;
  Address_counter : Word;
  Data_Byte : Byte;

begin



  // LED-Weiss_AT24C512 ein
  SetBit(LED_Weiss_AT24C512);
  LED_Weiss_AT24C512_Zaehler:=LED_Weiss_AT24C512_Zaehler_Ende;




  // Start-Sequenz
  SDA_Clear;

  SCL_Clear;


  // I2C Address
  Address_ACK:=I2C_Device_Address_to_Write(ADR_AT24C512);

  Data_Byte:=High(Flash_Address);

  Address_ACK:=ds_Shift_Out(Data_Byte);

  Data_Byte:=Low(Flash_Address);

  Address_ACK:=ds_Shift_Out(Data_Byte);



  for Address_counter:=0 to 127 do

    Data_Byte:=Sektor_Puffer[Address_counter];
    Address_ACK:=ds_Shift_Out(Data_Byte);

  endfor;


  // Stop-Sequenz
  SCL_Set;
  SDA_Set;


  // *******

  // Start-Sequenz
  SDA_Clear;
  SCL_Clear;

  // Acknowledge Polling (Datenblatt Seite 11)
  Address_ACK:=I2C_Device_Address_to_Write(ADR_AT24C512);


  // Stop-Sequenz
  SCL_Set;

  SDA_Set;


  while Address_ACK=false do

    // Start-Sequenz
    SDA_Clear;
    SCL_Clear;

    // I2C Address
    Address_ACK:=I2C_Device_Address_to_Write(ADR_AT24C512);

    // Stop-Sequenz
    SCL_Set;

    SDA_Set;


  endwhile;



end AT_Write_Data_Array_Wait;








procedure Format_DIR;
var
  b1 : Byte;
  Address_counter : Word;
  Address_Wert : Word;
begin


  // Sektor_Puffer mit E5H fuellen
  for Address_counter:=0 to 127 do
    Sektor_Puffer[Address_counter]:=$E5;
  endfor;


  // schreibe die 32 Sektoren fuer das Directory
  // mit E5 voll
  for Address_counter:=0 to 127 do

    Address_Wert:=Address_counter;

    // Address_Wert * 128
    SHL(Address_Wert);
    SHL(Address_Wert);
    SHL(Address_Wert);
    SHL(Address_Wert);
    SHL(Address_Wert);
    SHL(Address_Wert);
    SHL(Address_Wert);

    Flash_Address:=Address_Wert;

    // Schreiben der
    AT_Write_Data_Array_Wait;

  endfor;


end Format_DIR;






// ***************************************************





       {

procedure I2C_load_Address_List;
var
  Address_counter : Word;
  Address_value   : Byte;
  Address_line    : Byte;
  Address_ACK     : Boolean;
  b1              : Byte;
  b2              : Byte;

begin

  for Address_counter:=0 to 127 do

    // Start-Sequenz
    SDA_Clear;

    SCL_Clear;


    Address_value := LOW(Address_counter);

    // I2C Address
    Address_ACK:=I2C_Device_Address_to_Write(Address_value);

    if Address_ACK then
      Sektor_Puffer[Address_counter]:=1;
    else
      Sektor_Puffer[Address_counter]:=0;
    endif;

    // Stop-Sequenz
    SCL_Set;

    SDA_Set;

  endfor;


  // Address_Array to String_List


  SER0_String:='';
  WriteLn_SER0;

  wait_ms(1);

  SER0_String:='XX 00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F';
  WriteLn_SER0;

  wait_ms(1);

  Address_line:=0;
  SER0_String:='00';
  Address_value:=0;

  for Address_counter:=0 to 127 do

    b2:=Sektor_Puffer[Address_counter];

    if b2=1 then
      SER0_String:=SER0_String+' ';
      b1:=LOW(Address_counter);
      ByteToHexString(b1);
      SER0_String:=SER0_String+Byte_Hex_String;
    else
      SER0_String:=SER0_String+' ..';
    endif;

    INC(Address_line);

    if Address_line=16 then

      WriteLn_SER0;

      Address_line:=0;
      INC(Address_value);

      ByteToHexString(Address_value);

      SER0_String:=Byte_Hex_String;

    endif;

  endfor;

end I2C_load_Address_List;

        }



          {

procedure AT_zeige_ein_Byte;
var
  b1 : Byte;
begin

  b1:=AT_Data_Read_one_Byte;

  SER0_String:='Adresse=';
  WordToHexString(Flash_Address);
  SER0_String:=SER0_String+Word_Hex_String;
  WriteLn_SER0;


  SER0_String:='Daten=';
  ByteToHexString(b1);
  SER0_String:=SER0_String+Byte_Hex_String;
  WriteLn_SER0;


end AT_zeige_ein_Byte;

          }








// ***************************************
// *** Service-Routinen fuer das ARRAY ***
// ***************************************




procedure AT_fuelle_das_DatenArray(AnfangsWert, IncWert : Byte);
var
  Address_counter : Word;
  Daten_Wert   : Byte;


begin

  Daten_Wert:=AnfangsWert;

  for Address_counter:=0 to 127 do

    Sektor_Puffer[Address_counter]:=Daten_Wert;
    Daten_Wert:=Daten_Wert+IncWert;

  endfor;

end AT_fuelle_das_DatenArray;








// ************************************
// *** Service-Routinen zur Anzeige ***
// ************************************



            {

procedure AT_zeige_das_DatenArray;
var
  Address_counter : Word;
  Address_value   : Byte;
  Address_ACK     : Boolean;
  b1              : Byte;
  b2              : Byte;

begin


  // Daten_Array to String_List


  SER0_String:='';
  WriteLn_SER0;


  SER0_String:=' 00  01  02  03  04  05  06  07  08  09  0A  0B  0C  0D  0E  0F';
  WriteLn_SER0;


  b2:=0;
  SER0_String:='';
  Address_value:=0;

  for Address_counter:=0 to 127 do

    b1:=Sektor_Puffer[Address_counter];
    ByteToHexString(b1);
    SER0_String:=SER0_String+Byte_Hex_String;
    SER0_String:=SER0_String+'H ';

    INC(b2);

    if b2=16 then

      WriteLn_SER0;

      b2:=0;

      SER0_String:='';

    endif;

  endfor;

end AT_zeige_das_DatenArray;

            }




    {

procedure AT_zeige_das_DatenArray_Teil;
var
  Address_counter : Word;
  Address_value   : Byte;
  Address_ACK     : Boolean;
  b1              : Byte;
  b2              : Byte;

begin


  // Daten_Array to String_List


  SER0_String:='';
  WriteLn_SER0;


  SER0_String:=' 00  01  02  03  04  05  06  07  08  09  0A  0B  0C  0D  0E  0F';
  WriteLn_SER0;


  b2:=0;
  SER0_String:='';
  Address_value:=0;

  for Address_counter:=0 to 15 do

    b1:=Sektor_Puffer[Address_counter];
    ByteToHexString(b1);
    SER0_String:=SER0_String+Byte_Hex_String;
    SER0_String:=SER0_String+'H ';

    INC(b2);

    if b2=16 then

      WriteLn_SER0;

      b2:=0;

      SER0_String:='';

    endif;

  endfor;

  SER0_String:=' ';
  WriteLn_SER0;

end AT_zeige_das_DatenArray_Teil;

       }


         {

procedure AT_Read_mehrere_Arrays;
var
  Address_counter : Word;
  Address_Wert : Word;
begin

  SER0_String:='Start';
  WriteLn_SER0;

  for Address_counter:=0 to 511 do
     Address_Wert:=Address_counter;
     SHL(Address_Wert);
     SHL(Address_Wert);
     SHL(Address_Wert);
     SHL(Address_Wert);
     SHL(Address_Wert);
     SHL(Address_Wert);
     SHL(Address_Wert);

     Flash_Address:=Address_Wert;

     AT_Read_Data_Array;

  endfor;

  SER0_String:='Stop';
  WriteLn_SER0;


end AT_Read_mehrere_Arrays;

       }




