
// ****************************
// *** ITP3-Client-Routinen ***
// *** Software-Teil        ***
// ****************************


CONST




  // Befehlscode fuer ITP3
  ITP3_WHO_AM_I    : Byte = 01;
  ITP3_ERROR_CODE  : Byte = 02;

  ITP3_CONOUT  : Byte = 03;
  ITP3_CONST   : Byte = 04;
  ITP3_CONIN   : Byte = 05;

  ITP3_STD     : Byte = 06;
  ITP3_RD_SEC  : Byte = 07;
  ITP3_WR_SEC  : Byte = 08;

  ITP3_SOUND   : Byte = 09;
  ITP3_READER  : Byte = 10;





procedure ITP3_Client_Init;
begin

  // alle Datenleitungen auf Eingang schalten
  ClearBit(D0_D);
  ClearBit(D1_D);
  ClearBit(D2_D);
  ClearBit(D3_D);

  // Device Select auf Eingang
  ClearBit(DS_D);

  // DR auf Eingang
  ClearBit(DR_D);


end ITP3_Client_Init;







// Byte einlesen


function ITP3_Halbbyte_einlesen : Byte;
var
  b1 : Byte;
begin

  // warten, solange bis der Server das
  // zweite Halbbyte sendet
  SetBit(LED_Blau);
  while BitHigh(DS_E) do
  endwhile;
  ClearBit(LED_Blau);

  b1:=%00000000;

  // D0 einlesen
  if BitHigh(D0_E) then
    b1:=b1 or %00000001;
  endif;

  // D1 einlesen
  if BitHigh(D1_E) then
    b1:=b1 or %00000010;
  endif;

  // D2 einlesen
  if BitHigh(D2_E) then
    b1:=b1 or %00000100;
  endif;

  // D3 einlesen
  if BitHigh(D3_E) then
    b1:=b1 or %00001000;
  endif;

  // jetzt wird dem Server mitgeteilt,
  // das die Daten uebernommen wurden
  ClearBit(DR_A);

  // warten solange bis der Server die
  // Bestaetigung angenommen hat
  SetBit(LED_Blau);
  while BitLow(DS_E) do
  endwhile;
  ClearBit(LED_Blau);

  // DR auf High (Grundeinstellung)
  SetBit(DR_A);

  Result:=b1;

end ITP3_Halbbyte_einlesen;




function ITP3_Byte_einlesen : Byte;
var
  b1 : Byte;
  Result_Byte : Byte;
begin

  // DR auf High (Grundeinstellung)
  SetBit(DR_A);
  // DR auf Ausgang - sendebereit -
  SetBit(DR_D);

  Result_Byte:=ITP3_Halbbyte_einlesen;


  b1:=ITP3_Halbbyte_einlesen;

  // DR wieder auf Eingang fuer die anderen Clients
  ClearBit(DR_D);

  SHL(b1);
  SHL(b1);
  SHL(b1);
  SHL(b1);

  Result_Byte:=Result_Byte+b1;

  Result:=Result_Byte;

end ITP3_Byte_einlesen;












// Halbbyte ausgeben

procedure ITP3_Halbbyte_ausgeben(H_Byte : Byte);
var
  b1 : Byte;
begin

  // CB

  b1:=H_Byte and %00000001;

  // D0 ausgeben
  if b1=0 then
    ClearBit(D0_A);
  else
    SetBit(D0_A);
  endif;


  b1:=H_Byte and %00000010;

  // D1 ausgeben
  if b1=0 then
    ClearBit(D1_A);
  else
    SetBit(D1_A);
  endif;


  b1:=H_Byte and %00000100;

  // D2 ausgeben
  if b1=0 then
    ClearBit(D2_A);
  else
    SetBit(D2_A);
  endif;


  b1:=H_Byte and %00001000;

  // D3 ausgeben
  if b1=0 then
    ClearBit(D3_A);
  else
    SetBit(D3_A);
  endif;


  // jetzt wird dem Server mitgeteilt,
  // das die Daten zur Uebernahme
  // anliegen
  ClearBit(DR_A);

  // CC

  // warten solange bis der Server die
  // Daten eingelesen hat und dies
  // bestaetigt hat
  SetBit(LED_Blau);
  while BitLow(DS_E) do
  endwhile;
  ClearBit(LED_Blau);

  // CE

  // DR auf High (Grundeinstellung)
  SetBit(DR_A);

  // CF

end ITP3_Halbbyte_ausgeben;




procedure ITP3_Byte_ausgeben(H_Byte : Byte);
var
  b1 : Byte;

begin

  // DR auf High (Grundeinstellung)
  SetBit(DR_A);
  // DR auf Ausgang - Uebernahme der Steuerleitung
  SetBit(DR_D);


  // CA

  SetBit(LED_Blau);
  while BitHigh(DS_E) do
  endwhile;
  ClearBit(LED_Blau);

  // CB

  // alle Datenleitungen auf Ausgang schalten
  SetBit(D0_D);
  SetBit(D1_D);
  SetBit(D2_D);
  SetBit(D3_D);

  b1:=H_Byte and %00001111;
  ITP3_Halbbyte_ausgeben(b1);

  SHR(H_Byte);
  SHR(H_Byte);
  SHR(H_Byte);
  SHR(H_Byte);

  // warten, solange bis der Server das
  // untere Halbbyte gelesen hat
  SetBit(LED_Blau);
  while BitHigh(DS_E) do
  endwhile;
  ClearBit(LED_Blau);

  ITP3_Halbbyte_ausgeben(H_Byte);

  // DR wieder auf Eingang fuer die anderen Clients
  ClearBit(DR_D);

  // alle Datenleitungen auf Eingang schalten
  ClearBit(D0_D);
  ClearBit(D1_D);
  ClearBit(D2_D);
  ClearBit(D3_D);

end ITP3_Byte_ausgeben;








{

  Konfiguration WHO_ARE_YOU - Byte
  D7 - CONOUT
  D6 - CONIN
  D5 - LIST
  D4 - SOUND
  D3 - DIVE_D
  D2 - DIVE_E
  D1 - READER
  D0 - N.N.

}

procedure ITP3_PRG_WHO_AM_I;
var
  DATA_Byte : Byte;
begin

  // DRIVE
  DATA_Byte:=%00001000;
  ITP3_Byte_ausgeben(DATA_Byte);

  WHO_AM_I_Schleife:=false;

end ITP3_PRG_WHO_AM_I;








// Umwandeln von STD in Sektor_Nr und Spur_Nr
procedure ITP3_PRG_STD;
var
  DATA_Byte : Byte;
  DATA_Word : Word;
begin

  // SECTOR
  DATA_Byte:=ITP3_Byte_einlesen;
  STD_Sektor_Nr:=ByteToWord(DATA_Byte);

  // TRACK_L
  DATA_Byte:=ITP3_Byte_einlesen;
  DATA_Word:=ByteToWord(DATA_Byte);

  // TRACK_H
  DATA_Byte:=ITP3_Byte_einlesen;
  STD_Spur_Nr:=ByteToWord(DATA_Byte);
  STD_Spur_Nr:=STD_Spur_Nr*256;
  STD_Spur_Nr:=STD_Spur_Nr+DATA_Word;

  // DISKNO wird derzeit ignoriert
  DATA_Byte:=ITP3_Byte_einlesen;

  // Umwandeln von SPT=128 in SPT=32, da
  // ein 4K-Sektor gleich 32 Sektoren*128 Byte hat
  DATA_Word:=STD_Sektor_Nr and %0000000001000000;
  SHL(STD_Spur_Nr);
  if DATA_Word>0 then
    STD_Spur_Nr:=STD_Spur_Nr or %0000000000000001;
  endif;

  DATA_Word:=STD_Sektor_Nr and %0000000000100000;
  SHL(STD_Spur_Nr);
  if DATA_Word>0 then
    STD_Spur_Nr:=STD_Spur_Nr or %0000000000000001;
  endif;

  STD_Sektor_Nr:=STD_Sektor_Nr and %0000000000011111;

end ITP3_PRG_STD;










procedure ITP3_PRG_RD_SEC;
var
  DATA_Byte : Byte;
begin

  // LED-Gruen ein
  SetBit(LED_Gruen);
  // LED-Gelb aus
  ClearBit(LED_Gelb);
  LED_Gruen_Zaehler:=LED_Gruen_Zaehler_Ende;


  // Lesen des Sektors in den Sektor_Puffer
  FM_Read_Data_Array;


  for Sektor_Puffer_Zeiger:=0 to 127 do

    DATA_Byte:=Sektor_Puffer[Sektor_Puffer_Zeiger];
    ITP3_Byte_ausgeben(DATA_Byte);

  endfor;


end ITP3_PRG_RD_SEC;





procedure ITP3_PRG_WR_SEC;
var
  DATA_Byte : Byte;
begin


  // LED-Rot ein
  SetBit(LED_Rot);
  // LED_Gruen aus
  ClearBit(LED_Gelb);
  LED_Rot_Zaehler:=LED_Rot_Zaehler_Ende;



  FM_Test_Verschieben_Spur;




  for Sektor_Puffer_Zeiger:=0 to 127 do

    DATA_Byte:=ITP3_Byte_einlesen;
    Sektor_Puffer[Sektor_Puffer_Zeiger]:=DATA_Byte;

  endfor;

  FM_Write_Data_Array;



end ITP3_PRG_WR_SEC;












procedure ITP3_Server_Abfrage;
var
  CODE_Byte : Byte;
begin

  if BitLow(DS_E) then

    CODE_Byte:=ITP3_Byte_einlesen;

    case CODE_Byte of

      ITP3_WHO_AM_I : ITP3_PRG_WHO_AM_I;
      |

      ITP3_STD    : ITP3_PRG_STD;
      |

      ITP3_RD_SEC : ITP3_PRG_RD_SEC;
      |

      ITP3_WR_SEC : ITP3_PRG_WR_SEC;
      |

    endcase;

  //  Anzeige_des_Fehlercodes;

  endif;

end ITP3_Server_Abfrage;







