

var

  // Steuerleitungen fuer den W25Q64_EEPROM
  // ==========================================


  EEPROM_CS_A[PortB,2] : bit;   // Ausgang - Chip Select an PIN 1 des 25VF032B80_EEPROM
  EEPROM_MISO_E[PINB,4] : bit;  // Eingang - MISO an PIN 2 des 25VF032B80_EEPROM
  EEPROM_MOSI_A[PortB,3] : bit; // Ausgang - MOSI an PIN 5 des 25VF032B80_EEPROM
  EEPROM_SCK_A[PortB,5] : bit;  // Ausgang - SCK an PIN 6 des 25VF032B80_EEPROM

  EEPROM_CS_D[DDRB,2] : bit;
  EEPROM_MISO_D[DDRB,4] : bit;
  EEPROM_MOSI_D[DDRB,3] : bit;
  EEPROM_SCK_D[DDRB,5] : bit;




  one_Byte, SPI_Byte,
  ADR_Byte_2, ADR_Byte_1, ADR_Byte_0 : Byte;


  lfd_Nummer : Byte;




procedure Init_GPIO;
begin

  // Steuerleitungen auf Ausgang
  SetBit(EEPROM_CS_D);

  // SPI-Leitungen auf Ausgang
  SetBit(EEPROM_MOSI_D);
  SetBit(EEPROM_SCK_D);

  // SPI-Leitungen auf Eingang
  SetBit(EEPROM_MISO_D);



  // LowActive Steuerleitungen auf Hight
  SetBit(EEPROM_CS_A);

  // Datenleitungen auf Low
  ClearBit(EEPROM_MOSI_A);
  SetBit(EEPROM_SCK_A);

  lfd_Nummer:=19;

end Init_GPIO;







procedure Init_SPI;
var
  Init_SPI_Byte : Byte;
begin

  {
 Bit 7 — SPIE: SPI Interrupt Enable
  Wenn dieses Bit mit einer 1 beschrieben wird,
  ist der SPI-Interrupt freigegeben.
  Der Interrupt wird ausgeführt, wenn das I-Bit im SREG gesetzt ist
  und das SPIF Flag im SPSR Register gesetzt wird.

 Bit 6 — SPE: SPI Enable
  Wenn dieses Bit mit einer 1 beschrieben wird,
  ist das SPI freigegeben. Es muss gesetzt werden um alle
  SPI-Operationen freizugeben.

 Bit 5 — DORD: Data Order
  Wenn dieses Bit mit einer 1 beschrieben ist,
  werden die Daten mit dem LSB voran übertragen.
  Wenn dieses Bit gelöscht ist, werden die Daten mit dem
  MSB voran übertragen.

 Bit 4 — MSTR: Master / Slave Select
  Wenn dieses Bit mit einer 1 beschrieben ist,
  ist der Mastermodus ausgewählt, wenn es gelöscht ist,
  arbeitet die SPI im Slave-Modus.
  Wenn SS als Eingang konfiguriert ist und im Mastermodus
  auf Low gezogen wird, wird das MSTR-Bit gelöscht und SPIF
  im SPSR wird gesetzt.
  Der Anwender muss dann das MSTR-Bit wieder setzen
  um den Mastermodus zu reaktivieren.

 Bit 3 — CPOL: Clock Polatity
  Wenn dieses Bit mit einer 1 beschrieben ist,
  ist SCK auf High, wenn er nicht benutzt wird.
  Ist das Bit gelöscht, so ist SCK auf Low, wenn er nicht benutzt wird.

   CPOL  Vorderflanke Rückflanke
     0     steigend    fallend
     1     fallend    steigend

 Bit 2 — CPHA: Clock Phase
  Dieses Bit legt fest, ob die Daten mit der
  Vorder- oder Rückflanke von SCK abgetastet werden.
  Zusammengefasst ist die CPHA-Funktion wie folgt:

   CPHA  Vorderflanke Rückflanke
     0     abtasten    setzen
     1      setzen    abtasten

 Bit 1, 0 — SPR1, SPR0: SPI Clock Rate Select 1 und 0
  Dieses beiden Bits kontrollieren die Taktrate an SCK
  des Bausteins, wenn er im Mastermodus arbeitet.
  SPR1 und SPR0 haben keine Auswirkungen im Slavemodus.

  Bit7 = 0 -> kein Interrupt
  Bit6 = 1 -> SPI aktiviert
  Bit5 = 0 -> MSB first
  Bit4 = 1 -> SPI arbeitet als Master
  Bit3 = 0 -> CPOL: Clock Polatity -> Vorderflanke steigend
  Bit2 = 0 -> CPHA: Clock Phase -> Vorderflanke setzen

  Mode 0 = CPOL=0 CPAH=0

  Bit1, 0 = 1 -> SPR1, SPR0: SPI Clock Rate Select 1 und 0
              0 1 -> f(OSC) / 16

  }
//  SPCR := %01011000;

  SPCR := %01011100;

  // Verdopplung der Taktrate
  SPSR:=SPSR or %00000001;


end Init_SPI;





// ************************************************
// ** Basis Prozeduren fuer Lesen und Schreiben ***
// ************************************************


// schreibt das SPI_Byte in den EEPROM
procedure SPI_Write_Byte;
begin

  SPDR:=SPI_Byte;

  SetBit(LED_BLAU);

ASM;

Wait_Transmit:
  ; Wait for transmission complete
  in r16, SPSR
  sbrs r16, SPIF
  rjmp Wait_Transmit

ENDASM;

  ClearBit(LED_BLAU);

end SPI_Write_Byte;





// liest ein SPI_Byte aus dem EEPROM
procedure SPI_Read_Byte;
begin

  SetBit(LED_BLAU);

  SPDR:=$FF;

ASM;

Wait_Transmit3:
  ; Wait for transmission complete
  in r16, SPSR
  sbrs r16, SPIF
  rjmp Wait_Transmit3

ENDASM;

  SPI_Byte:=SPDR;

  ClearBit(LED_BLAU);


end SPI_Read_Byte;





// *****************************************************







// *********************************
// ***    FLASH Read (Lesen)     ***
// *********************************





// Read_StatusReg1 -> Ergebnis in SPI_Byte
procedure FLASH_Read_StatusReg1;
begin

  ClearBit(EEPROM_CS_A);

  SPI_Byte:=$05;
  SPI_Write_Byte;

  SPI_Read_Byte;

  SetBit(EEPROM_CS_A);

end FLASH_Read_StatusReg1;






procedure FLASH_Read_StatusReg2;
begin

  ClearBit(EEPROM_CS_A);

  SPI_Byte:=$35;
  SPI_Write_Byte;

  SPI_Read_Byte;

  SetBit(EEPROM_CS_A);

end FLASH_Read_StatusReg2;










// liest ein Byte auf der Adresse ADR_Byte_2 ADR_Byte_1 ADR_Byte_0
// und schreibt den Wert in das SPI_Byte
procedure FLASH_Read_JEDEC;
begin

  ClearBit(EEPROM_CS_A);

  SPI_Byte:=$9F;
  SPI_Write_Byte;

  SPI_Read_Byte;
  ADR_Byte_2:=SPI_Byte;

  SPI_Read_Byte;
  ADR_Byte_1:=SPI_Byte;

  SPI_Read_Byte;
  ADR_Byte_0:=SPI_Byte;

  SetBit(EEPROM_CS_A);

end FLASH_Read_JEDEC;











// liest ein Byte auf der Adresse ADR_Byte_2 ADR_Byte_1 ADR_Byte_0
// und schreibt den Wert in das SPI_Byte
procedure W25Q64_Read_one_Byte;
begin

  ClearBit(EEPROM_CS_A);

  SPI_Byte:=$03;
  SPI_Write_Byte;

  SPI_Byte:=ADR_Byte_2;
  SPI_Write_Byte;
  SPI_Byte:=ADR_Byte_1;
  SPI_Write_Byte;
  SPI_Byte:=ADR_Byte_0;
  SPI_Write_Byte;

  SPI_Read_Byte;

  SetBit(EEPROM_CS_A);

end W25Q64_Read_one_Byte;








// liest einen Sektor in den Sektor_Puffer
procedure W25Q64_Read_one_Sector;
begin



  // LED-Weiss_W25Q64 ein
  SetBit(LED_Weiss_W25Q64);
  LED_Weiss_W25Q64_Zaehler:=LED_Weiss_W25Q64_Zaehler_Ende;




  ClearBit(EEPROM_CS_A);

  SPI_Byte:=$03;
  SPI_Write_Byte;

  SPI_Byte:=ADR_Byte_2;
  SPI_Write_Byte;
  SPI_Byte:=ADR_Byte_1;
  SPI_Write_Byte;
  SPI_Byte:=ADR_Byte_0;
  SPI_Write_Byte;

  for Sektor_Puffer_Zeiger:=0 to 127 do

    SPI_Read_Byte;
    Sektor_Puffer[Sektor_Puffer_Zeiger]:=SPI_Byte;

  endfor;

  SetBit(EEPROM_CS_A);

end W25Q64_Read_one_Sector;




// *********************************
//     FLASH Write
// *********************************




// schreibt das SPI_Byte auf die Adresse ADR_Byte_2 ADR_Byte_1 ADR_Byte_0
procedure FLASH_Page_Program_one_Byte;
var
  PP_Byte : Byte;
begin

  PP_Byte:=SPI_Byte;

  ClearBit(EEPROM_CS_A);
  SPI_Byte:=$06;
  SPI_Write_Byte;
  SetBit(EEPROM_CS_A);


  ClearBit(EEPROM_CS_A);
  SPI_Byte:=$02;
  SPI_Write_Byte;

  SPI_Byte:=ADR_Byte_2;
  SPI_Write_Byte;
  SPI_Byte:=ADR_Byte_1;
  SPI_Write_Byte;
  SPI_Byte:=ADR_Byte_0;
  SPI_Write_Byte;

  SPI_Byte:=PP_Byte;
  SPI_Write_Byte;

  SetBit(EEPROM_CS_A);

end FLASH_Page_Program_one_Byte;






// schreibt den Sektor_Puffer (128 Byte) auf die Adresse ADR_Byte_2 ADR_Byte_1 ADR_Byte_0
procedure W25Q64_Write_one_Sector;
begin


  // LED-Weiss_W25Q64 ein
  SetBit(LED_Weiss_W25Q64);
  LED_Weiss_W25Q64_Zaehler:=LED_Weiss_W25Q64_Zaehler_Ende;


  ClearBit(EEPROM_CS_A);
  SPI_Byte:=$06;
  SPI_Write_Byte;
  SetBit(EEPROM_CS_A);


  ClearBit(EEPROM_CS_A);
  SPI_Byte:=$02;
  SPI_Write_Byte;

  SPI_Byte:=ADR_Byte_2;
  SPI_Write_Byte;
  SPI_Byte:=ADR_Byte_1;
  SPI_Write_Byte;
  SPI_Byte:=ADR_Byte_0;
  SPI_Write_Byte;


  for Sektor_Puffer_Zeiger:=0 to 127 do

    SPI_Byte:=Sektor_Puffer[Sektor_Puffer_Zeiger];
    SPI_Write_Byte;

  endfor;

  SetBit(EEPROM_CS_A);

end W25Q64_Write_one_Sector;








// schreibt den Sektor_Puffer (128 Byte) auf die Adresse ADR_Byte_2 ADR_Byte_1 ADR_Byte_0
procedure FLASH_Sektor_Puffer_Fill_Data;
begin

  for Sektor_Puffer_Zeiger:=0 to 127 do

    Sektor_Puffer[Sektor_Puffer_Zeiger]:=lfd_Nummer;
    lfd_Nummer:=lfd_Nummer+1;

  endfor;

  lfd_Nummer:=lfd_Nummer+13;

end FLASH_Sektor_Puffer_Fill_Data;











// *********************************
//     FLASH Loeschen
// *********************************


// The Sector-Erase instruction clears all bits
// in the selected 4 KByte sector to FFH
procedure FLASH_Sector_Erase;
begin


  ClearBit(EEPROM_CS_A);
  SPI_Byte:=$06;
  SPI_Write_Byte;
  SetBit(EEPROM_CS_A);


  ClearBit(EEPROM_CS_A);

  SPI_Byte:=$20;
  SPI_Write_Byte;

  SPI_Byte:=ADR_Byte_2;
  SPI_Write_Byte;
  SPI_Byte:=ADR_Byte_1;
  SPI_Write_Byte;
  SPI_Byte:=ADR_Byte_0;
  SPI_Write_Byte;

  SetBit(EEPROM_CS_A);


  ClearBit(EEPROM_CS_A);
  SetBit(LED_Rot);

  SPI_Byte:=$05;
  SPI_Write_Byte;

  SPI_Read_Byte;
  SPI_Byte:=SPI_Byte and %00000001;

  while SPI_Byte>0 do

    SPI_Byte:=$05;
    SPI_Write_Byte;

    SPI_Read_Byte;
    SPI_Byte:=SPI_Byte and %00000001;

  endwhile;

  ClearBit(LED_Rot);
  SetBit(EEPROM_CS_A);

end FLASH_Sector_Erase;




// The Chip-Erase instruction clears all bits in the device to FFH.
// dauert etwa 30 Sekunden !!!
procedure FLASH_Chip_Erase;
begin


  ClearBit(EEPROM_CS_A);
  SPI_Byte:=$06;
  SPI_Write_Byte;
  SetBit(EEPROM_CS_A);


  ClearBit(EEPROM_CS_A);
  SPI_Byte:=$C7;
  SPI_Write_Byte;
  SetBit(EEPROM_CS_A);


  ClearBit(EEPROM_CS_A);
  SetBit(LED_Rot);

  SPI_Byte:=$05;
  SPI_Write_Byte;

  SPI_Read_Byte;
  SPI_Byte:=SPI_Byte and %00000001;

  while SPI_Byte>0 do

    SPI_Byte:=$05;
    SPI_Write_Byte;

    SPI_Read_Byte;
    SPI_Byte:=SPI_Byte and %00000001;

  endwhile;

  ClearBit(LED_Rot);
  SetBit(EEPROM_CS_A);


end FLASH_Chip_Erase;







// *********************************
// ***   UART WriteLn Ausgabe    ***
// *********************************


          {

procedure UART_WriteLn_SPI_byte;
begin

   ByteToHexString(SPI_byte);
   SER0_String:='SPI_byte='+Byte_Hex_String;
   WriteLn_SER0;

end UART_WriteLn_SPI_byte;

          }


           {
procedure UART_WriteLn_ADR_Byte_0_bis_3;
begin

   ByteToHexString(ADR_Byte_2);
   SER0_String:='ADR_Byte_2='+Byte_Hex_String;
   WriteLn_SER0;

   ByteToHexString(ADR_Byte_1);
   SER0_String:='ADR_Byte_1='+Byte_Hex_String;
   WriteLn_SER0;

   ByteToHexString(ADR_Byte_0);
   SER0_String:='ADR_Byte_0='+Byte_Hex_String;
   WriteLn_SER0;

end UART_WriteLn_ADR_Byte_0_bis_3;
           }



            {
procedure UART_WriteLn_Sektor_Puffer;
var
  lfd_Zaehler : Byte;
begin

  SER0_String:='';
  lfd_Zaehler:=0;

  for Sektor_Puffer_Zeiger:=0 to 127 do

    SPI_byte:=Sektor_Puffer[Sektor_Puffer_Zeiger];
    ByteToHexString(SPI_byte);
    SER0_String:=SER0_String+Byte_Hex_String;
    SER0_String:=SER0_String+'H ';

    lfd_Zaehler:=lfd_Zaehler+1;

    if lfd_Zaehler=16 then

      WriteLn_SER0;

      lfd_Zaehler:=0;
      SER0_String:='';

    endif;

  endfor;

end UART_WriteLn_Sektor_Puffer;

        }









